/*
 * cdaudio.c - Binding for the cdaudio library
 *
 *  Written by FUJIWARA Teruyoshi (tf@dsl.gr.jp)
 *  I put this program in public domain.  Use it as you like.
 *
 *  $Id: $
 */

#include "cdaudio_.h"

static void discinfo_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
{
    Scm_Printf(out, "#,(discinfo %d %d %d %d %d %d)",
               SCM_DISCINFO_DI(obj).disc_present,
               SCM_DISCINFO_DI(obj).disc_mode,
               SCM_DISCINFO_DI(obj).disc_current_track,
	       SCM_DISCINFO_DI(obj).disc_first_track,
	       SCM_DISCINFO_DI(obj).disc_total_tracks,
	       0);
}

static void volume_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
{
    Scm_Printf(out, "#,(volume %d %d %d %d)",
               SCM_VOLUME_V(obj).vol_front.left,
               SCM_VOLUME_V(obj).vol_front.right,
               SCM_VOLUME_V(obj).vol_back.left,
	       SCM_VOLUME_V(obj).vol_back.right);
}

SCM_DEFINE_BUILTIN_CLASS(Scm_DiscInfoClass, discinfo_print, NULL, NULL, NULL, NULL);
SCM_DEFINE_BUILTIN_CLASS(Scm_VolumeClass, volume_print, NULL, NULL, NULL, NULL);

ScmObj Scm_MakeDiscInfo(struct disc_info di)
{
  ScmDiscInfo *d;
  int i;

  d = SCM_NEW(ScmDiscInfo);
  SCM_SET_CLASS(d, SCM_CLASS_DISCINFO);
  SCM_DISCINFO_DI(d) = di;

  return SCM_OBJ(d);
}

ScmObj Scm_MakeVolume(struct disc_volume vol)
{
  ScmVolume *v;

  v = SCM_NEW(ScmVolume);
  SCM_SET_CLASS(v, SCM_CLASS_VOLUME);
  SCM_VOLUME_V(v) = vol;

  return SCM_OBJ(v);
}

/*
 * Module initialization function.
 * This is called when cdaudio.so is dynamically loaded into gosh.
 */
ScmObj Scm_Init_cdaudio(void)
{
    ScmModule *mod;

    SCM_INIT_EXTENSION(cdaudio);
    mod = SCM_MODULE(SCM_FIND_MODULE("cdaudio", TRUE));

    Scm_InitBuiltinClass(&Scm_DiscInfoClass, "<discinfo>",
                         NULL, sizeof(ScmDiscInfo)/sizeof(ScmObj),
                         mod);
    Scm_InitBuiltinClass(&Scm_VolumeClass, "<volume>",
                         NULL, sizeof(ScmVolume)/sizeof(ScmObj),
                         mod);
 
    Scm_Init_cdaudiolib(mod);
}
